package org.eclipse.jdt.internal.compiler.ast;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.internal.compiler.ast.ASTNode;
import org.eclipse.jdt.internal.compiler.ast.Annotation;
import org.eclipse.jdt.internal.compiler.ast.ArrayInitializer;
import org.eclipse.jdt.internal.compiler.ast.Expression;
import org.eclipse.jdt.internal.compiler.ast.MemberValuePair;
import org.eclipse.jdt.internal.compiler.ast.QualifiedTypeReference;
import org.eclipse.jdt.internal.compiler.ast.SingleMemberAnnotation;
import org.eclipse.jdt.internal.compiler.ast.SingleTypeReference;
import org.eclipse.jdt.internal.compiler.ast.TypeReference;
import org.eclipse.jdt.internal.compiler.impl.Constant;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeConstants;

public class ContainerAnnotation extends SingleMemberAnnotation {
  private Annotation[] containees;
  
  private ArrayInitializer memberValues;
  
  public ContainerAnnotation(Annotation repeatingAnnotation, ReferenceBinding containerAnnotationType, BlockScope scope) {
    char[][] containerTypeName = containerAnnotationType.compoundName;
    if (containerTypeName.length == 1) {
      this.type = (TypeReference)new SingleTypeReference(containerTypeName[0], 0L);
    } else {
      this.type = (TypeReference)new QualifiedTypeReference(containerTypeName, new long[containerTypeName.length]);
    } 
    this.sourceStart = repeatingAnnotation.sourceStart;
    this.sourceEnd = repeatingAnnotation.sourceEnd;
    this.resolvedType = (TypeBinding)containerAnnotationType;
    this.recipient = repeatingAnnotation.recipient;
    this.containees = new Annotation[0];
    this.memberValue = (Expression)(this.memberValues = new ArrayInitializer());
    addContainee(repeatingAnnotation);
  }
  
  public void addContainee(Annotation repeatingAnnotation) {
    int length = this.containees.length;
    System.arraycopy(this.containees, 0, this.containees = new Annotation[length + 1], 0, length);
    this.containees[length] = repeatingAnnotation;
    this.memberValues.expressions = (Expression[])this.containees;
    repeatingAnnotation.setPersistibleAnnotation((length == 0) ? this : null);
  }
  
  public TypeBinding resolveType(BlockScope scope) {
    if (this.compilerAnnotation != null)
      return this.resolvedType; 
    this.constant = Constant.NotAConstant;
    ReferenceBinding containerAnnotationType = (ReferenceBinding)this.resolvedType;
    if (!containerAnnotationType.isValidBinding())
      containerAnnotationType = (ReferenceBinding)containerAnnotationType.closestMatch(); 
    Annotation repeatingAnnotation = this.containees[0];
    ReferenceBinding repeatingAnnotationType = (ReferenceBinding)repeatingAnnotation.resolvedType;
    if (!repeatingAnnotationType.isDeprecated() && isTypeUseDeprecated((TypeBinding)containerAnnotationType, (Scope)scope))
      scope.problemReporter().deprecatedType((TypeBinding)containerAnnotationType, (ASTNode)repeatingAnnotation); 
    checkContainerAnnotationType((ASTNode)repeatingAnnotation, scope, containerAnnotationType, repeatingAnnotationType, true);
    this.resolvedType = (TypeBinding)(containerAnnotationType = repeatingAnnotationType.containerAnnotationType());
    if (!this.resolvedType.isValidBinding())
      return this.resolvedType; 
    MethodBinding[] methods = containerAnnotationType.methods();
    MemberValuePair pair = memberValuePairs()[0];
    for (int i = 0, length = methods.length; i < length; i++) {
      MethodBinding method = methods[i];
      if (CharOperation.equals(method.selector, TypeConstants.VALUE)) {
        pair.binding = method;
        pair.resolveTypeExpecting(scope, method.returnType);
      } 
    } 
    this.compilerAnnotation = scope.environment().createAnnotation((ReferenceBinding)this.resolvedType, computeElementValuePairs());
    return this.resolvedType;
  }
}
